import edu.neu.ccs.*;
import edu.neu.ccs.gui.*;
import edu.neu.ccs.util.*;
import edu.neu.ccs.filter.*;
import edu.neu.ccs.console.*;
import edu.neu.ccs.pedagogy.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;
import java.lang.reflect.*;

public class HanoiView extends DisplayPanel {
    
    /** The panel width. */
    protected int width  = 1000;
    
    /** The panel height. */
    protected int height =  200;
    
    /** The horizontal positions of the centers of the three towers. */
    protected int[] center = new int[] { 160, 500, 840 };
    
    /** The smallest block size. */
    protected int diskBlock = 20;
    
    /** The jump between the bottom on one disk and the bottom of the next. */
    protected int diskJump = 25;
    
    /** The model for this view. */
    protected HanoiModel model = null;
    
    /** A helper rectangle object. */
    private Rectangle2D.Double rectangle = new Rectangle2D.Double();
    
    /** Constructor. */
    public HanoiView(HanoiModel model) {
        this.model = model;
        
        setPreferredSize(new Dimension(width, height));
    }
    
    /** The paintComponent method. */
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        
        g2.setPaint(Color.white);
        rectangle.setFrame(0, 0, width, height);
        g2.fill(rectangle);
        g2.setPaint(Color.black);
        
        if (model == null)
            return;
        
        for (int tower = 0; tower <= 2; tower++)
            for (int level = 0; level < HanoiModel.maxDisks; level++) {
                int size = model.getSize(tower, level);
                
                if (size > 0) {
                    int u = center[tower];
                    int half = size * diskBlock;
                    int x = u - half;
                    
                    int y = height - level * diskJump - diskBlock;
                    
                    rectangle.setFrame(x, y, 2 * half, diskBlock);
                    
                    g2.fill(rectangle);
                }
            }
    }
}